# Generated by Django 5.0 on 2023-12-22 09:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
        ("emodjis", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="emoji",
            name="nsfw",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="emoji",
            name="team",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="emojis",
                to="auth.group",
            ),
        ),
    ]



"""Admin view for API stats."""

import csv
import zipfile
from django.contrib import admin
from django.utils.timezone import now
from django.http import HttpResponse

from .models import Emoji


class ExportCsvMixin:
    def export_as_csv(self, request, queryset):
        meta = self.model._meta

        response = HttpResponse(content_type="text/csv")
        response[
            "Content-Disposition"
        ] = f"attachment; filename={meta}-{now().strftime('%Y%m%d')}.zip"
        writer = csv.writer(response)

        writer.writerow(self.model.EXPORT_FIELDS)
        for obj in queryset:
            writer.writerow(
                [getattr(obj, field) for field in self.model.EXPORT_FIELDS]
            )

        return response

    export_as_csv.short_description = "Export metadata as CSV"


class ExportZipMixin:
    def export_as_zip(self, request, queryset):
        meta = self.model._meta
        response = HttpResponse(content_type="application/zip")
        response[
            "Content-Disposition"
        ] = f"attachment; filename={meta}-{now().strftime('%Y%m%d')}.zip"
        zip_file = zipfile.ZipFile(response, "w")
        for obj in queryset:
            zip_file.writestr(
                f"{obj.name}.gif",
                obj.image,
            )

        return response

    export_as_zip.short_description = "Export metadata and files as ZIP"


class EmojiAdmin(admin.ModelAdmin, ExportCsvMixin, ExportZipMixin):
    actions = ["export_as_csv", "export_as_zip"]

    list_per_page = 20
    list_display = (
        "name",
        "team",
        "private",
        "nsfw",
        "uses",
        "created_at",
        "created_by",
        "deleted_at",
        "deleted_by",
    )

    fields = (
        "name",
        "team",
        "nsfw",
        "private",
        "uses",
        "created_by",
        "created_at",
        "deleted_at",
        "deleted_by",
    )
    list_filter = ("name", "uses", "nsfw", "private")
    search_fields = ("name",)
    readonly_fields = (
        "created_at",
        "created_by",
        "deleted_at",
        "deleted_by",
        "image",
        "uses",
    )

    change_list_template = "charts_change_list.html"

    def get_queryset(self, request):
        if request.user.is_superuser:
            return Emoji.admin.all()
        else:
            return Emoji.updates.nsfw(user=request.user)

    def changelist_view(self, request, extra_context=None):
        response = super().changelist_view(request, extra_context)
        try:
            filtered_query_set = response.context_data["cl"].queryset
        except KeyError:
            return response
        usage = filtered_query_set.values("name", "image", "uses").order_by(
            "-uses", "name"
        )[:10]
        extra_context = dict(usage=usage)
        response.context_data.update(extra_context)
        return response

    def changeform_view(
        self, request, object_id=None, form_url="", extra_context=None
    ):
        if request.GET.get("export_csv", False):
            export_queryset = self.get_queryset(request).filter(pk=object_id)
            return self.export_as_csv(request, export_queryset)
        if request.GET.get("export_zip", False):
            export_queryset = self.get_queryset(request).filter(pk=object_id)
            return self.export_as_csv(request, export_queryset)
        return super().changeform_view(
            request, object_id, form_url, extra_context
        )

    def has_add_permission(self, request, obj=None):
        return False

    def has_change_permission(self, request, obj=None):
        if request.user.is_superuser:
            return True
        if obj:
            return request.user == obj.created_by or request.user.is_superuser
        return False


admin.site.register(Emoji, EmojiAdmin)



import base64

from rest_framework.test import APIClient
from django.test import TestCase
from django.core.files.uploadedfile import SimpleUploadedFile
from django.contrib.auth.models import User, Group

from emodjis.models import Emoji

username = "test"
email = "test@test.com"
password = "test"

teams = ["team1", "team2"]

test_name = "alert"
test_image = "emodjis/tests/alert.gif"
test_wrong_image = "emodjis/tests/blank.png"

emoticon_list_url = "/emoticons"
emoticon_detail_url = "/emoticon/{name}"


class TestEmoticonAPI(TestCase):
    def setUp(self):
        User.objects.create_user(
            username=username, email=email, password=password
        )
        User.objects.create_user(
            username=username + "2", email=email, password=password
        )
        for team in teams:
            Group.objects.create(name=team)

        self.test_emoji = self.create_test_image(name=test_name)
        self.private_emoji = self.create_test_image(
            name=test_name + "1", private=True
        )
        self.nsfw_emoji = self.create_test_image(
            name=test_name + "2", nsfw=True
        )
        self.team1_emoji = self.create_test_image(
            name=test_name + "3", team=Group.objects.get(name=teams[0])
        )
        self.team2_emoji = self.create_test_image(
            name=test_name + "4", team=Group.objects.get(name=teams[1])
        )

    @staticmethod
    def create_test_image(
        name: str, private: bool = False, nsfw: bool = False, team: str = None
    ):
        with open(test_image, "rb") as f:
            user = User.objects.get(username=username)
            emoji = Emoji.objects.create(
                name=name,
                image=f.read(),
                created_by=user,
                private=private,
                nsfw=nsfw,
                team=team,
            )
            return emoji

    def test_retrieve_emoticon(self):
        name = test_name
        client = APIClient()
        response = client.get(emoticon_detail_url.format(name=name))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.headers["Content-Type"], "image/gif")
        self.assertIsNotNone(response.content)

    def test_unknown_emoticon(self):
        name = test_name + "_unknown"
        client = APIClient()
        response = client.get(emoticon_detail_url.format(name=name))
        self.assertEqual(response.status_code, 404)
        self.assertEqual(response.headers["Content-Type"], "application/json")

    def test_create_emoticon(self):
        name = "new_emoticon"
        client = APIClient()
        basic_auth = base64.b64encode(
            f"{username}:{password}".encode("latin1")
        )
        client.credentials(
            HTTP_AUTHORIZATION="Basic " + basic_auth.decode("utf-8")
        )
        with open(test_image, "rb") as f:
            image = SimpleUploadedFile(
                f.name, f.read(), content_type="image/gif"
            )
            response = client.post(
                emoticon_detail_url.format(name=name),
                data={"image": image},
            )
        self.assertEqual(response.status_code, 201)
        self.assertEqual(response.headers["Content-Type"], "application/json")
        self.assertIsNotNone(response.json())

    def test_create_emoticon_unknown_format(self):
        name = "wrong_format"
        client = APIClient()
        basic_auth = base64.b64encode(
            f"{username}:{password}".encode("latin1")
        )
        client.credentials(
            HTTP_AUTHORIZATION="Basic " + basic_auth.decode("utf-8")
        )
        response = client.post(
            emoticon_detail_url.format(name=name),
            {"image": open(test_wrong_image, "rb")},
        )
        self.assertEqual(response.status_code, 400)

    def test_create_emoticon_unauthenticated(self):
        name = "new_emoticon"
        client = APIClient()
        response = client.post(
            emoticon_detail_url.format(name=name),
            files={"image": open(test_image, "rb")},
        )
        self.assertEqual(response.status_code, 401)

    def test_delete_emoticon(self):
        name = "to_delete"
        self.create_test_image(name=name)
        client = APIClient()
        basic_auth = base64.b64encode(
            f"{username}:{password}".encode("latin1")
        )
        client.credentials(
            HTTP_AUTHORIZATION="Basic " + basic_auth.decode("utf-8")
        )
        response = client.delete(emoticon_detail_url.format(name=name))
        self.assertEqual(response.status_code, 204)

    def test_delete_emoticon_unauthenticated(self):
        name = "to_delete"
        self.create_test_image(name=name)
        client = APIClient()
        response = client.delete(emoticon_detail_url.format(name=name))
        self.assertEqual(response.status_code, 401)

    def test_delete_emoticon_wrong_user(self):
        name = "to_delete"
        self.create_test_image(name=name)
        client = APIClient()
        basic_auth = base64.b64encode(
            f"{username+'2'}:{password}".encode("latin1")
        )
        client.credentials(
            HTTP_AUTHORIZATION="Basic " + basic_auth.decode("utf-8")
        )
        response = client.delete(emoticon_detail_url.format(name=name))
        self.assertEqual(response.status_code, 404)

    def test_list_emoticons(self):
        client = APIClient()
        response = client.get(emoticon_list_url)
        print("list response", response.json())
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.headers["Content-Type"], "application/json")
        self.assertIsNotNone(response.json())

    def test_list_filter_team(self):
        client = APIClient()
        response = client.get(emoticon_list_url + "?team=" + teams[0])
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.headers["Content-Type"], "application/json")
        print("team filter", response.json())
        self.assertEqual(response.json()[0]["name"], self.team1_emoji.name)

    def test_list_filter_nsfw(self):
        client = APIClient()
        response = client.get(emoticon_list_url + "?nsfw=true")
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.headers["Content-Type"], "application/json")
        has_nsfw = False
        print("nsfw filter", response.json())
        for item in response.json():
            if item["name"] == self.nsfw_emoji.name:
                has_nsfw = True
        self.assertTrue(has_nsfw)

    def test_list_filter_private(self):
        client = APIClient()
        basic_auth = base64.b64encode(
            f"{username}:{password}".encode("latin1")
        )
        client.credentials(
            HTTP_AUTHORIZATION="Basic " + basic_auth.decode("utf-8")
        )
        response = client.get(emoticon_list_url + "?private=true")
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.headers["Content-Type"], "application/json")
        has_private = False
        print("private filter", response.json())
        for item in response.json():
            if item["name"] == self.private_emoji.name:
                has_private = True
        self.assertTrue(has_private)

    def test_list_filter_private_unauthenticated(self):
        client = APIClient()
        response = client.get(emoticon_list_url + "?private=true")
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.headers["Content-Type"], "application/json")
        has_private = False
        print("private filter", response.json())
        for item in response.json():
            if item["name"] == self.private_emoji.name:
                has_private = True
        self.assertFalse(has_private)

    def test_retrieve_schema(self):
        url = "/schema/?format=json"
        client = APIClient()
        response = client.get(url)
        assert response.status_code == 200
        assert response.headers["Content-Type"] in [
            "application/json",
            "application/vnd.oai.openapi+json",
        ]
        assert response.json() is not None



"""Emoticons serializer."""
import base64
from django.conf import settings
from django.utils.translation import gettext_lazy as _
from .models import Emoji
from rest_framework import serializers


class EmojiFilterSerializer(serializers.ModelSerializer):
    name = serializers.CharField(label=_("Search by name"), required=False)
    team = serializers.CharField(
        label=_("Search by team name"), required=False
    )
    nsfw = serializers.BooleanField(
        label=_("Show NSFW emoticons"), required=False
    )
    private = serializers.BooleanField(
        label=_("Show private emoticons only"), required=False
    )

    class Meta:
        model = Emoji
        fields = ["name", "team", "nsfw", "private"]


class EmojiSerializer(serializers.ModelSerializer):
    b64image = serializers.SerializerMethodField()
    url = serializers.SerializerMethodField()

    class Meta:
        model = Emoji
        fields = "__all__"

    def get_b64image(self, obj):
        return base64.b64encode(obj.image).decode("utf-8")

    def get_url(self, obj):
        if server_url := getattr(settings, "SERVER_URL", None):
            return f"{server_url}/emoticon/{obj.name}"
        request = self.context.get("request")
        if request:
            host = request.META.get("HTTP_HOST")
            return f"{request.scheme}://{host}/emoticon/{obj.name}"
        else:
            return f"emoticon/{obj.name}"


class EmojiListSerializer(serializers.ModelSerializer):
    url = serializers.SerializerMethodField()

    class Meta:
        model = Emoji
        fields = ["name", "url"]

    def get_url(self, obj):
        if server_url := getattr(settings, "SERVER_URL", None):
            return f"{server_url}/emoticon/{obj.name}"
        request = self.context.get("request")
        if request:
            host = request.META.get("HTTP_HOST")
            return f"{request.scheme}://{host}/emoticon/{obj.name}"
        else:
            return f"emoticon/{obj.name}"


class EmojiDestroySerializer(serializers.ModelSerializer):
    class Meta:
        model = Emoji
        fields = []


class EmojiCreateSerializer(serializers.ModelSerializer):
    image = serializers.FileField(label=_("File"), required=False)

    class Meta:
        model = Emoji
        fields = ["image", "private", "nsfw", "team"]



from django_unicorn.components import UnicornView


class SearchView(UnicornView):
    search = ""
    nsfw = False
    private = False

    def updated_search(self, query):
        self.parent.load_emojis(
            search=query, nsfw=self.nsfw, private=self.private
        )
        self.parent.force_render = True

    def updated_nsfw(self, query):
        self.nsfw = query
        self.parent.load_emojis(
            search=self.search, nsfw=query, private=self.private
        )
        self.parent.force_render = True

    def updated_private(self, query):
        self.private = query
        self.parent.load_emojis(
            search=self.search, nsfw=self.nsfw, private=query
        )
        self.parent.force_render = True



from django_unicorn.components import UnicornView
from django.contrib.auth import login, authenticate, logout


class LoginView(UnicornView):
    user = None
    username = None
    password = None
    authenticated = False

    def mount(self):
        self.authenticated = self.request.user.is_authenticated

    def connect(self):
        if user := authenticate(
            request=self.request,
            username=self.username,
            password=self.password,
        ):
            login(request=self.request, user=user)
            self.user = user
            self.authenticated = user.is_authenticated
        self.parent.load_emojis()
        self.parent.force_render = True

    def disconnect(self):
        self.user = None
        self.authenticated = False
        logout(request=self.request)
        self.parent.load_emojis()
        self.parent.force_render = True



# grid.py
from django_unicorn.components import UnicornView
from django.core.paginator import Paginator
from ..models import Emoji
from ..serializers import EmojiSerializer

PAGE_SIZE = 100


class EmojiGridView(UnicornView):
    name_search = ""
    emodjis = None
    page_range = 0
    page = 1

    def mount(self):
        self.load_emojis()

    def load_emojis(self, search="", page=1, nsfw=False, private=False):
        self.name_search = search
        if not self.request.user.is_authenticated:
            emodjis = Emoji.objects.filter(nsfw=False)
        else:
            if nsfw:
                emodjis = Emoji.objects.nsfw(user=self.request.user)
            else:
                emodjis = Emoji.objects.sfw(user=self.request.user)
        if not private:
            emodjis = emodjis.filter(private=False)
        if search:
            emodjis = emodjis.filter(name__icontains=search).order_by("name")
        else:
            emodjis = emodjis.filter(
                name__icontains=self.name_search
            ).order_by("name")
        p = Paginator(emodjis, PAGE_SIZE)
        self.page_range = list(p.page_range)
        self.page = page
        self.emodjis = EmojiSerializer(
            p.page(page).object_list,
            many=True,
            context={"request": self.request},
        ).data

    def get_page_range(self):
        return self.page_range



"""Emoticons viewsets"""
import uuid
import logging
from PIL import Image, UnidentifiedImageError
from django.http import HttpResponse
from django.utils.timezone import now
from django.utils.translation import gettext as _
from django_filters.rest_framework import DjangoFilterBackend
from drf_spectacular.utils import extend_schema
from drf_spectacular.openapi import OpenApiTypes
from rest_framework import viewsets
from rest_framework.response import Response
from rest_framework.parsers import FormParser, MultiPartParser
from .models import Emoji
from .serializers import (
    EmojiSerializer,
    EmojiListSerializer,
    EmojiDestroySerializer,
    EmojiCreateSerializer,
    EmojiFilterSerializer,
)
from .permissions import IsOwnerOrReadOnly
from .filters import EmojiFilter

BASE_WIDTH = 130


class EmojiViewSet(viewsets.ModelViewSet):
    """Emoji viewset."""

    lookup_field = "name"
    parser_classes = (MultiPartParser, FormParser)
    filter_backends = [DjangoFilterBackend]
    filterset_class = EmojiFilter

    def get_queryset(self):
        qs = Emoji.objects.sfw(user=self.request.user)
        if self.action in [
            "list",
            "retrieve",
        ] and self.request.query_params.get("nsfw", False):
            qs = Emoji.objects.nsfw(user=self.request.user)
        if self.action in ["create", "update", "partial_update", "destroy"]:
            qs = Emoji.updates.filter(created_by=self.request.user)
        qs = self.filter_queryset(qs)
        return qs

    queryset = Emoji.objects.all()
    serializer_class = EmojiSerializer
    permission_classes = (IsOwnerOrReadOnly,)

    @staticmethod
    def handle_file_upload(uploaded_file) -> bytes:
        upload = uploaded_file
        if upload:
            tmp_file_name = str(uuid.uuid4())
            gif_file_name = tmp_file_name + ".gif"
            with open(f"/tmp/{tmp_file_name}", "wb") as tmp_file:
                for chunk in upload.chunks():
                    tmp_file.write(chunk)
            content = Image.open(f"/tmp/{tmp_file_name}")
            width_ratio = BASE_WIDTH / float(content.size[0])
            hsize = int((float(content.size[1]) * float(width_ratio)))
            content = content.resize(
                (BASE_WIDTH, hsize), Image.Resampling.LANCZOS
            )
            content.save(f"/tmp/{gif_file_name}", format="GIF")
            content.close()
            with open(f"/tmp/{gif_file_name}", "rb") as f:
                content = f.read()
            return content

    def get_serializer_class(self):
        match self.action:
            case "list":
                return EmojiListSerializer
            case "retrieve":
                return EmojiSerializer
            case "create":
                return EmojiCreateSerializer
            case "destroy":
                return EmojiDestroySerializer

        return super().get_serializer_class()

    @extend_schema(
        summary=_("List emoticons"),
        description=_("""Filter emoticons."""),
        responses={200: OpenApiTypes.BINARY, 404: OpenApiTypes.OBJECT},
        parameters=[EmojiFilterSerializer],
        tags=[
            "emoticons",
        ],
        methods=["GET"],
    )
    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)

    @extend_schema(
        summary=_("Retrieve emoticon"),
        description=_("""Retrieve visible emoticon image by name."""),
        responses={200: OpenApiTypes.BINARY, 404: OpenApiTypes.OBJECT},
        tags=[
            "emoticons",
        ],
        methods=["GET"],
    )
    def retrieve(self, request, name, *args, **kwargs):
        try:
            instance = self.get_queryset().get(pk=name)
            instance.uses = instance.uses + 1
            instance.save()
            return HttpResponse(instance.image, content_type="image/gif")
        except Emoji.DoesNotExist:
            return Response(
                {"detail": _("Emoji does not exist.")},
                status=404,
                content_type="application/json",
            )

    @extend_schema(
        summary=_("Create a new emoticon"),
        description=_(
            """
            Create a new icon with or without icon.

            Upload can happen later,
            but only icons with content will be visible.
            """
        ),
        request=EmojiCreateSerializer,
        responses={
            201: EmojiSerializer,
            400: OpenApiTypes.OBJECT,
            500: OpenApiTypes.OBJECT,
        },
        tags=[
            "emoticons",
        ],
        methods=["POST"],
    )
    def create(self, request, name, *args, **kwargs):
        if self.get_queryset().filter(pk=name).exists():
            return Response(
                {"detail": _("Emoji already exists.")},
                status=400,
                content_type="application/json",
            )
        create_serializer = self.get_serializer_class()(data=request.data)
        if not create_serializer.is_valid():
            return Response(
                create_serializer.errors,
                status=400,
                content_type="application/json",
            )
        if uploaded_file := request.FILES.get("image"):
            try:
                content = self.handle_file_upload(uploaded_file=uploaded_file)
            except UnidentifiedImageError:
                logger = logging.getLogger("django")
                logger.error("Uploaded file is not an image.")
                return Response(
                    {"detail": _("Uploaded file is not an image.")},
                    status=400,
                    content_type="application/json",
                )
            except IOError:
                logger = logging.getLogger("django")
                logger.error("I/O error.")
                return Response(
                    {"detail": _("I/O error.")},
                    status=500,
                    content_type="application/json",
                )
            emoticon = create_serializer.save(
                name=name, created_by=request.user, image=content
            )
        else:
            emoticon = create_serializer.save(
                name=name, created_by=request.user
            )
        serializer = EmojiSerializer(emoticon)
        return Response(
            serializer.data, status=201, content_type="application/json"
        )

    @extend_schema(
        summary=_("Delete an emoticon"),
        description=_("""Delete an emoticon you own."""),
        request=EmojiDestroySerializer,
        responses={204: OpenApiTypes.NONE, 400: OpenApiTypes.OBJECT},
        tags=[
            "emoticons",
        ],
        methods=["DELETE"],
    )
    def destroy(self, request, name, *args, **kwargs):
        try:
            instance = self.get_queryset().get(pk=name)
            instance.deleted_by = request.user
            instance.deleted_at = now()
            instance.save(update_fields=["deleted_by", "deleted_at"])
            return Response(status=204)
        except Emoji.DoesNotExist:
            return Response(
                {"detail": _("Emoji does not exist.")},
                status=404,
                content_type="application/json",
            )



from django_unicorn.components import UnicornView


class PaginationView(UnicornView):
    page = 1
    page_range = None

    def hydrate(self, *args, **kwargs):
        self.page_range = self.parent.get_page_range()
        self.page = self.parent.page

    def updated_page(self, query=1):
        self.page = query
        self.parent.load_emojis(page=query)
        self.page_range = self.parent.get_page_range()
        self.parent.force_render = True



# Generated by Django 5.0 on 2023-12-20 15:41

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Emoji",
            fields=[
                (
                    "name",
                    models.CharField(
                        max_length=255, primary_key=True, serialize=False
                    ),
                ),
                ("uses", models.IntegerField(default=0)),
                ("image", models.BinaryField(null=True)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("deleted_at", models.DateTimeField(blank=True, null=True)),
                (
                    "created_by",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="created_emojis",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "deleted_by",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="deleted_emojis",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]
